include "../glewpy.pxi"

cdef extern from "GL/glew.h":
   cdef enum:
      c_GLEW_MESA_pack_invert "GLEW_MESA_pack_invert"
      c_GLEW_MESA_resize_buffers "GLEW_MESA_resize_buffers"
      c_GLEW_MESA_window_pos "GLEW_MESA_window_pos"
      c_GLEW_MESA_ycbcr_texture "GLEW_MESA_ycbcr_texture"

def GLEW_MESA_pack_invert(): return c_GLEW_MESA_pack_invert
def GLEW_MESA_resize_buffers(): return c_GLEW_MESA_resize_buffers
def GLEW_MESA_window_pos(): return c_GLEW_MESA_window_pos
def GLEW_MESA_ycbcr_texture(): return c_GLEW_MESA_ycbcr_texture

# -------------------------- GL_MESA_pack_invert -------------------------- #
GL_PACK_INVERT_MESA = 0x8758

# ------------------------- GL_MESA_resize_buffers ------------------------ #
cdef extern from "GL/glew.h":
   void c_glResizeBuffersMESA "glResizeBuffersMESA"()

def glResizeBuffersMESA():
   if c_GLEW_MESA_resize_buffers:
      c_glResizeBuffersMESA()
   else:
      raise GlewpyError('GL_MESA_resize_buffers', 'glResizeBuffersMESA')

# --------------------------- GL_MESA_window_pos -------------------------- #
cdef extern from "GL/glew.h":
   void c_glWindowPos2dMESA "glWindowPos2dMESA"(GLdouble x, GLdouble y)
   void c_glWindowPos2dvMESA "glWindowPos2dvMESA"(GLdouble* p)
   void c_glWindowPos2fMESA "glWindowPos2fMESA"(GLfloat x, GLfloat y)
   void c_glWindowPos2fvMESA "glWindowPos2fvMESA"(GLfloat* p)
   void c_glWindowPos2iMESA "glWindowPos2iMESA"(GLint x, GLint y)
   void c_glWindowPos2ivMESA "glWindowPos2ivMESA"(GLint* p)
   void c_glWindowPos2sMESA "glWindowPos2sMESA"(GLshort x, GLshort y)
   void c_glWindowPos2svMESA "glWindowPos2svMESA"(GLshort* p)
   void c_glWindowPos3dMESA "glWindowPos3dMESA"(GLdouble x, GLdouble y, GLdouble z)
   void c_glWindowPos3dvMESA "glWindowPos3dvMESA"(GLdouble* p)
   void c_glWindowPos3fMESA "glWindowPos3fMESA"(GLfloat x, GLfloat y, GLfloat z)
   void c_glWindowPos3fvMESA "glWindowPos3fvMESA"(GLfloat* p)
   void c_glWindowPos3iMESA "glWindowPos3iMESA"(GLint x, GLint y, GLint z)
   void c_glWindowPos3ivMESA "glWindowPos3ivMESA"(GLint* p)
   void c_glWindowPos3sMESA "glWindowPos3sMESA"(GLshort x, GLshort y, GLshort z)
   void c_glWindowPos3svMESA "glWindowPos3svMESA"(GLshort* p)
   void c_glWindowPos4dMESA "glWindowPos4dMESA"(GLdouble x, GLdouble y, GLdouble z, GLdouble)
   void c_glWindowPos4dvMESA "glWindowPos4dvMESA"(GLdouble* p)
   void c_glWindowPos4fMESA "glWindowPos4fMESA"(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
   void c_glWindowPos4fvMESA "glWindowPos4fvMESA"(GLfloat* p)
   void c_glWindowPos4iMESA "glWindowPos4iMESA"(GLint x, GLint y, GLint z, GLint w)
   void c_glWindowPos4ivMESA "glWindowPos4ivMESA"(GLint* p)
   void c_glWindowPos4sMESA "glWindowPos4sMESA"(GLshort x, GLshort y, GLshort z, GLshort w)
   void c_glWindowPos4svMESA "glWindowPos4svMESA"(GLshort* p)

def glWindowPos2dMESA(x, y):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos2dMESA(x, y)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2dMESA')

def glWindowPos2dvMESA(p):
   cdef GLdouble pos[2]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      c_glWindowPos2dvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2dvMESA')

def glWindowPos2fMESA(x, y):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos2fMESA(x, y)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2fMESA')

def glWindowPos2fvMESA(p):
   cdef GLfloat pos[2]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      c_glWindowPos2fvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2fvMESA')

def glWindowPos2iMESA(x, y):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos2iMESA(x, y)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2iMESA')

def glWindowPos2fiMESA(p):
   cdef GLint pos[2]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      c_glWindowPos2ivMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2ivMESA')

def glWindowPos2sMESA(x, y):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos2sMESA(x, y)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2sMESA')

def glWindowPos2svMESA(p):
   cdef GLshort pos[2]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      c_glWindowPos2svMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos2svMESA')

def glWindowPos3dMESA(x, y, z):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos3dMESA(x, y, z)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3dMESA')

def glWindowPos3dvMESA(p):
   cdef GLdouble pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      c_glWindowPos3dvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3dvMESA')

def glWindowPos3fMESA(x, y, z):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos3fMESA(x, y, z)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3fMESA')

def glWindowPos3fvMESA(p):
   cdef GLfloat pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      c_glWindowPos3fvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3fvMESA')

def glWindowPos3iMESA(x, y, z):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos3iMESA(x, y, z)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3iMESA')

def glWindowPos3fiMESA(p):
   cdef GLint pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      c_glWindowPos3ivMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3ivMESA')

def glWindowPos3sMESA(x, y, z):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos3sMESA(x, y, z)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3sMESA')

def glWindowPos3fsMESA(p):
   cdef GLshort pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      c_glWindowPos3svMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos3svMESA')

def glWindowPos4dMESA(x, y, z, w):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos4dMESA(x, y, z, w)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4dMESA')

def glWindowPos4dvMESA(p):
   cdef GLdouble pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      pos[3] = p[3]
      c_glWindowPos4dvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4dvMESA')

def glWindowPos4fMESA(x, y, z, w):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos4fMESA(x, y, z, w)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4fMESA')

def glWindowPos4fvMESA(p):
   cdef GLfloat pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      pos[3] = p[3]
      c_glWindowPos4fvMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4fvMESA')

def glWindowPos4iMESA(x, y, z, w):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos4iMESA(x, y, z, w)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4iMESA')

def glWindowPos4fiMESA(p):
   cdef GLint pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      pos[3] = p[3]
      c_glWindowPos4ivMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4ivMESA')

def glWindowPos4sMESA(x, y, z, w):
   if c_GLEW_MESA_window_pos:
      c_glWindowPos4sMESA(x, y, z, w)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4sMESA')

def glWindowPos4fsMESA(p):
   cdef GLshort pos[3]

   if c_GLEW_MESA_window_pos:
      pos[0] = p[0]
      pos[1] = p[1]
      pos[2] = p[2]
      pos[3] = p[3]
      c_glWindowPos4svMESA(pos)
   else:
      raise GlewpyError('GL_MESA_window_pos', 'glWindowPos4svMESA')

# ------------------------- GL_MESA_ycbcr_texture ------------------------- #
GL_UNSIGNED_SHORT_8_8_MESA = 0x85BA
GL_UNSIGNED_SHORT_8_8_REV_MESA = 0x85BB
GL_YCBCR_MESA = 0x8757



include "../glewpy.pxi"

cdef extern from "GL/glew.h":
   cdef enum:
      c_GLEW_IBM_cull_vertex "GLEW_IBM_cull_vertex"
      c_GLEW_IBM_multimode_draw_arrays "GLEW_IBM_multimode_draw_arrays"
      c_GLEW_IBM_rasterpos_clip "GLEW_IBM_rasterpos_clip"
      c_GLEW_IBM_static_data "GLEW_IBM_static_data"
      c_GLEW_IBM_texture_mirrored_repeat "GLEW_IBM_texture_mirrored_repeat"
      c_GLEW_IBM_vertex_array_lists "GLEW_IBM_vertex_array_lists"

def GLEW_IBM_cull_vertex(): return c_GLEW_IBM_cull_vertex
def GLEW_IBM_multimode_draw_arrays(): return c_GLEW_IBM_multimode_draw_arrays
def GLEW_IBM_rasterpos_clip(): return c_GLEW_IBM_rasterpos_clip
def GLEW_IBM_static_data(): return c_GLEW_IBM_static_data
def GLEW_IBM_texture_mirrored_repeat(): return c_GLEW_IBM_texture_mirrored_repeat
def GLEW_IBM_vertex_array_lists(): return c_GLEW_IBM_vertex_array_lists

# --------------------------- GL_IBM_cull_vertex -------------------------- #
GL_CULL_VERTEX_IBM = 103050

# ---------------------- GL_IBM_multimode_draw_arrays --------------------- #
cdef extern from "GL/glew.h":
   void c_glMultiModeDrawArraysIBM "glMultiModeDrawArraysIBM"(GLenum* mode, GLint *first, GLsizei *count, GLsizei primcount, GLint modestride)
   void c_glMultiModeDrawElementsIBM "glMultiModeDrawElementsIBM"(GLenum* mode, GLsizei *count, GLenum type, GLvoid * *indices, GLsizei primcount, GLint modestride)

def glMultiModeDrawArraysIBM(mode, first, count, primcount, modestride):
   cdef GLenum arg1[1]
   cdef GLint arg2[1]
   cdef GLsizei arg3[1]

   if c_GLEW_IBM_cull_vertex:
      arg1[0] = mode[0]
      arg2[0] = first[0]
      arg3[0] = count[0]
      c_glMultiModeDrawArraysIBM(arg1, arg2, arg3, primcount, modestride)
   else:
      raise GlewpyError('GL_IBM_multimode_draw_arrays', 'glMultiModeDrawArraysIBM')

# ------------------------- GL_IBM_rasterpos_clip ------------------------- #
GL_RASTER_POSITION_UNCLIPPED_IBM = 103010

# --------------------------- GL_IBM_static_data -------------------------- #
GL_ALL_STATIC_DATA_IBM = 103060
GL_STATIC_VERTEX_ARRAY_IBM = 103061

# --------------------- GL_IBM_texture_mirrored_repeat -------------------- #
GL_MIRRORED_REPEAT_IBM = 0x8370

# ----------------------- GL_IBM_vertex_array_lists ----------------------- #
GL_VERTEX_ARRAY_LIST_IBM = 103070
GL_NORMAL_ARRAY_LIST_IBM = 103071
GL_COLOR_ARRAY_LIST_IBM = 103072
GL_INDEX_ARRAY_LIST_IBM = 103073
GL_TEXTURE_COORD_ARRAY_LIST_IBM = 103074
GL_EDGE_FLAG_ARRAY_LIST_IBM = 103075
GL_FOG_COORDINATE_ARRAY_LIST_IBM = 103076
GL_SECONDARY_COLOR_ARRAY_LIST_IBM = 103077
GL_VERTEX_ARRAY_LIST_STRIDE_IBM = 103080
GL_NORMAL_ARRAY_LIST_STRIDE_IBM = 103081
GL_COLOR_ARRAY_LIST_STRIDE_IBM = 103082
GL_INDEX_ARRAY_LIST_STRIDE_IBM = 103083
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM = 103084
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM = 103085
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM = 103086
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM = 103087

cdef extern from "GL/glew.h":
   void c_glColorPointerListIBM "glColorPointerListIBM"(GLint size, GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glEdgeFlagPointerListIBM "glEdgeFlagPointerListIBM"(GLint stride, GLboolean ** pointer, GLint ptrstride)
   void c_glFogCoordPointerListIBM "glFogCoordPointerListIBM"(GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glIndexPointerListIBM "glIndexPointerListIBM"(GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glNormalPointerListIBM "glNormalPointerListIBM"(GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glSecondaryColorPointerListIBM "glSecondaryColorPointerListIBM"(GLint size, GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glTexCoordPointerListIBM "glTexCoordPointerListIBM"(GLint size, GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)
   void c_glVertexPointerListIBM "glVertexPointerListIBM"(GLint size, GLenum type, GLint stride, GLvoid ** pointer, GLint ptrstride)



# Include PyMem functions for all files
cdef extern from "Python.h":
   struct PyObject
   
   void* PyMem_Malloc(int n)
   void PyMem_Free(void *p)
   PyObject* PyList_New(int size)
   PyObject* PyString_FromStringAndSize(char*, int)

class GlewpyError(Exception):
   """Error for catching the use of non supported function calls"""
   def __init__(self, extname, funcname):
      """GlewpyError(GL Extension Name, Function Name)"""
      self.value = (extname, funcname)
   def __str__(self):
      return 'Non-supported extension "%s" for function "%s"!  Did you call glewInit?\n' % self.value

class GlewpyArrayError(Exception):
   """Error to be thrown if trying to use fast access on an expected
   iterable fails."""
   def __init__(self, funcname, index):
      """GlewpyArrayError(Function Name, Argument Index)"""
      self.value = (funcname, index)
   def __str__(self):
      return 'In method "%s", could not gain fast access to argument %i!\n' % self.value

# These types are pretty much used in every files
ctypedef unsigned int GLenum
ctypedef unsigned char GLboolean
ctypedef unsigned int GLbitfield
ctypedef signed char GLbyte
ctypedef short GLshort
ctypedef int GLint
ctypedef int GLsizei
ctypedef unsigned char GLubyte
ctypedef unsigned short GLushort
ctypedef unsigned int GLuint
ctypedef float GLfloat
ctypedef float GLclampf
ctypedef double GLdouble
ctypedef double GLclampd
ctypedef void GLvoid
ctypedef char GLchar

ctypedef char GLcharARB
ctypedef unsigned int GLhandleARB
   
ctypedef int ptrdiff_t
ctypedef ptrdiff_t GLsizeiptrARB
ctypedef ptrdiff_t GLintptrARB
ctypedef ptrdiff_t GLsizeiptr
ctypedef ptrdiff_t GLintptr

ctypedef unsigned short GLhalf



include "../glewpy.pxi"

cdef extern from "GL/glew.h":
   cdef enum:
      c_GL_S3_s3tc "GL_S3_s3tc"
      
def GL_S3_s3tc(): return c_GL_S3_s3tc

# ----------------------- GL_S3_s3tc ---------------------- #
GL_RGB_S3TC = 0x83A0
GL_RGB4_S3TC = 0x83A1
GL_RGBA_S3TC = 0x83A2
GL_RGBA4_S3TC = 0x83A3
GL_RGBA_DXT5_S3TC = 0x83A4
GL_RGBA4_DXT5_S3TC = 0x83A5



include "../glewpy.pxi"

cdef extern from "GL/glew.h":
   cdef enum:
      c_GL_REND_screen_coordinates "GL_REND_screen_coordinates"
      
def GL_REND_screen_coordinates(): return c_GL_REND_screen_coordinates

# ----------------------- GL_REND_screen_coordinates ---------------------- #
GL_SCREEN_COORDINATES_REND = 0x8490
GL_INVERTED_SCREEN_W_REND = 0x8491
